local Macronomy_EventFrame, events = CreateFrame("FRAME"), {}
local MacronomyAddonIsLoaded = false
local MacronomyMacrosAreLoaded = false
local MacronomyEnteredWorld = false
local MacronomyTimer = 0
local MacronomyInterval = .1
local MacronomyButtonUpdate = false

--------------------------------------------------------------------
function events:ADDON_LOADED(...)
	local addonName = ...
	if addonName == "Macronomy" then
		MacronomyAddonIsLoaded = true
		Macronomy_Init()
		Macronomy_EventFrame:UnregisterEvent("ADDON_LOADED")
	end
end

function events:UPDATE_MACROS(...)
	MacronomyMacrosAreLoaded = true
	Macronomy_Init()
	Macronomy_EventFrame:UnregisterEvent("UPDATE_MACROS")
end

function events:PLAYER_ENTERING_WORLD(...)
	MacronomyEnteredWorld = true
	Macronomy_Init()
	Macronomy_EventFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

--------------------------------------------------------------------
Macronomy_EventFrame:SetScript("OnEvent", function(self, event, ...)
	events[event](self, ...)
end)

Macronomy_EventFrame:SetScript("OnUpdate", function(self, elapsed)
	MacronomyTimer = MacronomyTimer + elapsed
	if MacronomyTimer > MacronomyInterval then
		Macronomy_SetButtons()
		MacronomyTimer = 0
	end
end)
--------------------------------------------------------------------
for i, v in pairs(events) do
	Macronomy_EventFrame:RegisterEvent(i)
end
--------------------------------------------------------------------
function Macronomy_Init()
	if MacronomyAddonIsLoaded and MacronomyMacrosAreLoaded and MacronomyEnteredWorld then
		MacronomyFrame:Hide()
		MacronomyFrame.name = "Macronomy"
		InterfaceOptions_AddCategory(MacronomyFrame);

		if not MacronomyByChar then MacronomyByChar = {} end
		MacronomyByChar.currentClass = UnitClass("player")
		MacronomyByChar.currentSpec = Macronomy_GetPlayerSpec()
		MacronomyByChar.currentCategory = "Default"
		MacronomyByChar.currentButton = "m1"
		if not MacronomyByChar.categoryInUse then MacronomyByChar.categoryInUse = 1 end

		if not Macronomy then
			Macronomy = {}
			for classNum = 1, getn(MacronomyClassList) do
				Macronomy[MacronomyClassList[classNum]] = {}

				for specNum = 1, getn(MacronomySpecList[MacronomyClassList[classNum]]) do
					Macronomy[MacronomyClassList[classNum]][MacronomySpecList[MacronomyClassList[classNum]][specNum]] = {}
					Macronomy[MacronomyClassList[classNum]][MacronomySpecList[MacronomyClassList[classNum]][specNum]].categories = {"Default"}
					Macronomy_InitCategory(MacronomyClassList[classNum], MacronomySpecList[MacronomyClassList[classNum]][specNum], "Default")
				end
			end

			DEFAULT_CHAT_FRAME:AddMessage("Macronomy variable initialized")
		end
		
		MacronomyMacroFrame_ebxMacro:SetTextInsets(8,5,5,16)
		MacronomyNewCategoryFrame_ebxNewCat:SetTextInsets(8,5,5,5)
		MacronomyExportFrame_scrollFrame_ebxExport:SetTextInsets(8,5,5,5)
		MacronomyImportFrame_scrollFrame_ebxImport:SetTextInsets(8,5,5,5)
		Macronomy_SetDropDowns()
		MacronomyButtonUpdate = true
		MacronomyFrame:RegisterForDrag("LeftButton")
		Macronomy.active = false
		DEFAULT_CHAT_FRAME:AddMessage("Macronomy initialized")
	end
end
--------------------------------------------------------------------
function Macronomy_GetPlayerSpec()
	local _, charSpec = GetSpecializationInfo(GetSpecialization())
	if not charSpec then charSpec = "NoSpec" end
	return charSpec
end
--------------------------------------------------------------------
function Macronomy_InitCategory(className, specName, categoryName)
	Macronomy[className][specName][categoryName] = {}
	for btnNum = 1, 12 do
		Macronomy[className][specName][categoryName][MacronomyButtonList[btnNum]] = ""
	end
	Macronomy[className][specName].categories[getn(Macronomy[className][specName].categories) + 1] = categoryName
end
--------------------------------------------------------------------
function Macronomy_SetDropDowns()
	Macronomy_setDropDownTo(MacronomyFrame_btnClass,MacronomyByChar.currentClass)
	Macronomy_setDropDownTo(MacronomyFrame_btnSpec,MacronomyByChar.currentSpec)
	Macronomy_setDropDownTo(MacronomyFrame_btnCategory,MacronomyByChar.currentCategory)
	Macronomy_setDropDownTo(MacronomyFrame_btnButton,MacronomyByChar.currentButton)
	Macronomy_setDropDownTo(MacronomyExportFrame_btnClass,MacronomyByChar.currentExportClass or "All")
	Macronomy_setDropDownTo(MacronomyExportFrame_btnSpec,MacronomyByChar.currentExportSpec or "All")
	Macronomy_setDropDownTo(MacronomyExportFrame_btnCategory,MacronomyByChar.currentExportCategory or "All")
	Macronomy_setDropDownTo(MacronomyExportFrame_btnButton,MacronomyByChar.currentExportButton or "All")

	if MacronomyByChar.currentCategory == "Default" then
		MacronomyFrame_btnCategoryRemove:Disable()
	else
		MacronomyFrame_btnCategoryRemove:Enable()
	end
	
	MacronomyMacroFrame_ebxMacro:SetText(Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec][MacronomyByChar.currentCategory][MacronomyByChar.currentButton] or "")
	Macronomy_SetCharacterCount()
	MacronomyMacroFrame_btnSave:Disable()
	--MacronomyMacroFrame_ebxMacro:SetFocus()
end
--------------------------------------------------------------------
function Macronomy_macroTextChanged()
	if MacronomyMacroFrame_ebxMacro:GetText() == Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec][MacronomyByChar.currentCategory][MacronomyByChar.currentButton] then
		MacronomyMacroFrame_btnSave:Disable()
	else
		MacronomyMacroFrame_btnSave:Enable()
	end
end
--------------------------------------------------------------------
function Macronomy_SetCharacterCount()
	local macroLength = MacronomyMacroFrame_ebxMacro:GetText():len()
	MacronomyMacroFrame_txtChars:SetText("Characters used: " .. macroLength .. " / 255")
end
--------------------------------------------------------------------
function Macronomy_escapePressed()
	Macronomy.active = false
	MacronomyFrame:Hide()
end
--------------------------------------------------------------------
function Macronomy_SetButtons()
	if MacronomyButtonUpdate and not UnitAffectingCombat("player") then
		-- Create/Set buttons to current Class/Spec/Category
		for btnNum = 1, 12 do
			local btnName = MacronomyButtonList[btnNum]
			if not GetMacroInfo(btnName) then CreateMacro(btnName, "INV_MISC_QUESTIONMARK", "#showtooltip", nil) end
			local charClass = UnitClass("player")
			local btnText = Macronomy[charClass][Macronomy_GetPlayerSpec()][Macronomy[charClass][Macronomy_GetPlayerSpec()].categories[MacronomyByChar.categoryInUse]][btnName]
			
			EditMacro(btnName,btnName, nil, btnText, 1, 1)
		end
		MacronomyButtonUpdate = false
	end
end
--------------------------------------------------------------------
function Macronomy_ButtonClick(btn)
	local btnName = btn:GetName()

	if btnName == "MacronomyFrame_btnX" then
		Macronomy.active = false
		MacronomyFrame:Hide()
	elseif btnName == "MacronomyFrame_btnCategoryAdd" then
		Macronomy_disableFilterSection()
		MacronomyMacroFrame:Hide()
		MacronomyNewCategoryFrame:Show()
	elseif btnName == "MacronomyFrame_btnCategoryRemove" then
		Macronomy_disableFilterSection()
		MacronomyMacroFrame:Hide()
		MacronomyDeleteCategoryFrame:Show()
		MacronomyDeleteCategoryFrame_txtConfirm:SetText("Are you sure you want to remove " .. MacronomyByChar.currentCategory .. " from " .. MacronomyByChar.currentSpec .. " " .. MacronomyByChar.currentClass)
	elseif btnName == "MacronomyMacroFrame_btnSave" then
		Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec][MacronomyByChar.currentCategory][MacronomyByChar.currentButton] = MacronomyMacroFrame_ebxMacro:GetText()
		MacronomyMacroFrame_btnSave:Disable()
		MacronomyButtonUpdate = true
	elseif btnName == "MacronomyNewCategoryFrame_btnOK" then
		Macronomy_createCategory(MacronomyNewCategoryFrame_ebxNewCat:GetText())
		Macronomy_enableFilterSection()
		MacronomyNewCategoryFrame_ebxNewCat:SetText("")
		MacronomyMacroFrame:Show()
		MacronomyNewCategoryFrame:Hide()
	elseif btnName == "MacronomyNewCategoryFrame_btnCancel" then
		Macronomy_enableFilterSection()
		MacronomyNewCategoryFrame_ebxNewCat:SetText("")
		MacronomyMacroFrame:Show()
		MacronomyNewCategoryFrame:Hide()
	elseif btnName == "MacronomyDeleteCategoryFrame_btnYes" then
		Macronomy_enableFilterSection()
		Macronomy_removeCategory()
		MacronomyMacroFrame:Show()
		MacronomyDeleteCategoryFrame:Hide()
	elseif btnName == "MacronomyDeleteCategoryFrame_btnNo" then
		Macronomy_enableFilterSection()
		MacronomyMacroFrame:Show()
		MacronomyDeleteCategoryFrame:Hide()
	elseif btnName == "MacronomyMacroFrame_btnImport" then
		MacronomyMacroFrame:Hide()
		MacronomyImportFrame_scrollFrame_ebxImport:SetText("")
		MacronomyImportFrame:Show()
		Macronomy_disableFilterSection()
	elseif btnName == "MacronomyImportFrame_btnImport" then
		Macronomy_ImportText()
		MacronomyMacroFrame:Show()
		MacronomyImportFrame:Hide()
		Macronomy_enableFilterSection()
	elseif btnName == "MacronomyImportFrame_btnCancel" then
		MacronomyMacroFrame:Show()
		MacronomyImportFrame:Hide()
		Macronomy_enableFilterSection()
	elseif btnName == "MacronomyMacroFrame_btnExport" then
		MacronomyByChar.currentExportClass = "All"
		MacronomyByChar.currentExportSpec = "All"
		MacronomyByChar.currentExportCategory = "All"
		MacronomyByChar.currentExportButton = "All"
		Macronomy_UpdateExportText()
		Macronomy_SetDropDowns()
		MacronomyMacroFrame:Hide()
		MacronomyExportFrame:Show()
		Macronomy_disableFilterSection()
	elseif btnName == "MacronomyExportFrame_btnOK" then
		MacronomyMacroFrame:Show()
		MacronomyExportFrame:Hide()
		Macronomy_enableFilterSection()
	end
end
--------------------------------------------------------------------
local MacronomyDropDownSize = {}
MacronomyDropDownSize["MacronomyFrame_btnClass"] = 120
MacronomyDropDownSize["MacronomyFrame_btnSpec"] = 120
MacronomyDropDownSize["MacronomyFrame_btnCategory"] = 120
MacronomyDropDownSize["MacronomyFrame_btnButton"] = 50
MacronomyDropDownSize["MacronomyExportFrame_btnClass"] = 120
MacronomyDropDownSize["MacronomyExportFrame_btnSpec"] = 120
MacronomyDropDownSize["MacronomyExportFrame_btnCategory"] = 120
MacronomyDropDownSize["MacronomyExportFrame_btnButton"] = 50

function Macronomy_dropMenuShow(self)
	UIDropDownMenu_Initialize(self, Macronomy_dropMenuInitialize);
	UIDropDownMenu_SetWidth(self,MacronomyDropDownSize[self:GetName()],3);
	UIDropDownMenu_JustifyText(self, "LEFT");
end

function Macronomy_dropMenuInitialize(btn)
	local listForBtn
	if btn:GetName() == "MacronomyFrame_btnClass" then
		listForBtn = MacronomyClassList
	elseif btn:GetName() == "MacronomyExportFrame_btnClass" then
		listForBtn = MacronomyExportClassList
	elseif btn:GetName() == "MacronomyFrame_btnSpec" then
		local currentClass = UnitClass("player")
		if MacronomyByChar then currentClass = MacronomyByChar.currentClass end
		listForBtn = MacronomySpecList[currentClass]
	elseif btn:GetName() == "MacronomyExportFrame_btnSpec" then
		local currentClass = "All"
		if MacronomyByChar then currentClass = MacronomyByChar.currentExportClass or "All" end
		listForBtn = MacronomyExportSpecList[currentClass]
	elseif btn:GetName() == "MacronomyFrame_btnCategory" then
		listForBtn = {"Default"}
		if Macronomy and MacronomyByChar then listForBtn = Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories end
	elseif btn:GetName() == "MacronomyExportFrame_btnCategory" then
		listForBtn = {"All"}
		if MacronomyByChar and MacronomyByChar.currentExportSpec ~= "All" then
			for catCount=1,getn(Macronomy[MacronomyByChar.currentExportClass][MacronomyByChar.currentExportSpec].categories) do
				listForBtn[catCount + 1] = Macronomy[MacronomyByChar.currentExportClass][MacronomyByChar.currentExportSpec].categories[catCount]
			end
		end
	elseif btn:GetName() == "MacronomyFrame_btnButton" then
		listForBtn = MacronomyButtonList
	elseif btn:GetName() == "MacronomyExportFrame_btnButton" then
		listForBtn = MacronomyExportButtonList
		if MacronomyByChar and MacronomyByChar.currentExportCategory == "All" then listForBtn = {"All"} end
	end

	Macronomy_populateButton(btn, listForBtn)
end

function Macronomy_populateButton(btn, listForBtn)
	if listForBtn then
		for listCount=1,getn(listForBtn) do
			Macronomy_addDropDownMenuBtn(listForBtn[listCount], btn)
		end
	end
end

function Macronomy_addDropDownMenuBtn(str, btn)
	local info = UIDropDownMenu_CreateInfo()
	info.text = str
	info.func = Macronomy_dropDownMenuBtnClick
	info.owner = btn
	UIDropDownMenu_AddButton(info)
end

function Macronomy_dropDownMenuBtnClick(self)
	local filtersChanged = false
	local exportFiltersChanged = false
	local parentFrame = self.owner;
	local selection = self:GetText();
	Macronomy_setDropDownTo(parentFrame, selection)
	
	if parentFrame == MacronomyFrame_btnClass and MacronomyByChar.currentClass ~= selection then
		MacronomyByChar.currentClass = selection
		MacronomyByChar.currentSpec = MacronomySpecList[MacronomyByChar.currentClass][1]
		if MacronomyByChar.currentClass == UnitClass("player") then
			MacronomyByChar.currentSpec = Macronomy_GetPlayerSpec()
		end
		MacronomyByChar.currentCategory = "Default"
		MacronomyByChar.currentButton = "m1"
		Macronomy_dropMenuInitialize(MacronomyFrame_btnClass)
		filtersChanged = true
	end

	if parentFrame == MacronomyFrame_btnSpec and MacronomyByChar.currentSpec ~= selection then
		MacronomyByChar.currentSpec = selection
		MacronomyByChar.currentCategory = "Default"
		MacronomyByChar.currentButton = "m1"
		filtersChanged = true
	end
	
	if parentFrame == MacronomyFrame_btnCategory and MacronomyByChar.currentCategory ~= selection then
		MacronomyByChar.currentCategory = selection
		filtersChanged = true
	end

	if parentFrame == MacronomyFrame_btnButton and MacronomyByChar.currentButton ~= selection then
		MacronomyByChar.currentButton = selection
		filtersChanged = true
	end
	
	if parentFrame == MacronomyExportFrame_btnClass and MacronomyByChar.currentExportClass ~= selection then
		MacronomyByChar.currentExportClass = selection
		MacronomyByChar.currentExportSpec = "All"
		MacronomyByChar.currentExportCategory = "All"
		MacronomyByChar.currentExportButton = "All"
		exportFiltersChanged = true
	end

	if parentFrame == MacronomyExportFrame_btnSpec and MacronomyByChar.currentExportSpec ~= selection then
		MacronomyByChar.currentExportSpec = selection
		MacronomyByChar.currentExportCategory = "All"
		MacronomyByChar.currentExportButton = "All"
		exportFiltersChanged = true
	end

	if parentFrame == MacronomyExportFrame_btnCategory and MacronomyByChar.currentExportCategory ~= selection then
		MacronomyByChar.currentExportCategory = selection
		MacronomyByChar.currentExportButton = "All"
		exportFiltersChanged = true
	end

	if parentFrame == MacronomyExportFrame_btnButton and MacronomyByChar.currentExportButton ~= selection then
		MacronomyByChar.currentExportButton = selection
		exportFiltersChanged = true
	end
	
	if exportFiltersChanged then Macronomy_UpdateExportText() end

	if filtersChanged or exportFiltersChanged then
		Macronomy_SetDropDowns()
	end
end

function Macronomy_setDropDownTo(btn, str)
	UIDropDownMenu_SetSelectedValue(btn, str)
	UIDropDownMenu_SetText(btn, str)
end
--------------------------------------------------------------------
function Macronomy_createCategory(newCat)
	local createNewCategory = true
	if newCat == "" or newCat == "Default" then createNewCategory = false end
	for catCount=1,getn(Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories) do
		if newCat == Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories[catCount] then createNewCategory = false end
	end

	if createNewCategory then
		Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories[getn(Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories) + 1] = newCat
		Macronomy_InitCategory(MacronomyByChar.currentClass, MacronomyByChar.currentSpec, newCat)
		MacronomyByChar.currentCategory = newCat

		Macronomy_SetDropDowns()
	end
end
--------------------------------------------------------------------
function Macronomy_shiftCategory()
	MacronomyByChar.categoryInUse = MacronomyByChar.categoryInUse + 1
	if MacronomyByChar.categoryInUse > getn(Macronomy[UnitClass("player")][Macronomy_GetPlayerSpec()].categories) then MacronomyByChar.categoryInUse = 1 end
	MacronomyButtonUpdate = true
	DEFAULT_CHAT_FRAME:AddMessage("Macros set to category: " .. Macronomy_GetPlayerSpec() .. " " .. UnitClass("player") .. " " .. Macronomy[UnitClass("player")][Macronomy_GetPlayerSpec()].categories[MacronomyByChar.categoryInUse])
end
--------------------------------------------------------------------
function Macronomy_removeCategory()
	if MacronomyByChar.currentCategory ~= "Default" then
		local isPastCategory = false
		local isCurrentRole = false
		local catTotalCount = getn(Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories)
		for catCount=1,catTotalCount do
			if isPastCategory then
				Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories[catCount - 1] = Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories[catCount]
			elseif MacronomyByChar.currentCategory == Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories[catCount] then
				if MacronomyByChar.currentClass == UnitClass("player") and MacronomyByChar.currentSpec == Macronomy_GetPlayerSpec() and MacronomyByChar.categoryInUse == catCount then
					MacronomyByChar.categoryInUse = 1
					MacronomyButtonUpdate = true
				end
				isPastCategory = true
			end
		end
		Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec].categories[catTotalCount] = nil
		Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec][MacronomyByChar.currentCategory] = nil
		MacronomyByChar.currentCategory = "Default"
		Macronomy_SetDropDowns()
	end
end
--------------------------------------------------------------------
function Macronomy_UpdateExportText()
	local exportText = ""

	for exportClassCount = 1, getn(MacronomyClassList) do
		if MacronomyByChar.currentExportClass == "All" or MacronomyByChar.currentExportClass == MacronomyClassList[exportClassCount] then
			local exportLoopClass = MacronomyClassList[exportClassCount]
			exportText = exportText .. (exportText ~= "" and "~~~~~" or "") .. exportLoopClass .. ":"

			local numberOfExportSpecs = 0
			for exportSpecCount = 1, getn(MacronomySpecList[MacronomyClassList[exportClassCount]]) do
				if MacronomyByChar.currentExportSpec == "All" or MacronomyByChar.currentExportSpec == MacronomySpecList[exportLoopClass][exportSpecCount] then
					local exportLoopSpec = MacronomySpecList[exportLoopClass][exportSpecCount]
					numberOfExportSpecs = numberOfExportSpecs + 1
					exportText = exportText .. (numberOfExportSpecs > 1 and "~~~~" or "") .. exportLoopSpec .. ":"
				
					local numberOfExportCategories = 0
					for exportCategoryCount = 1, getn(Macronomy[MacronomyClassList[exportClassCount]][exportLoopSpec].categories) do
						if MacronomyByChar.currentExportCategory == "All" or MacronomyByChar.currentExportCategory == Macronomy[exportLoopClass][exportLoopSpec].categories[exportCategoryCount] then
							local exportLoopCategory = Macronomy[exportLoopClass][exportLoopSpec].categories[exportCategoryCount]
							numberOfExportCategories = numberOfExportCategories + 1
							exportText = exportText .. (numberOfExportCategories > 1 and "~~~" or "") .. exportLoopCategory .. ":"

							local numberOfExportButtons = 0
							for exportButtonCount = 1, getn(MacronomyButtonList) do
								if MacronomyByChar.currentExportButton == "All" or MacronomyByChar.currentExportButton == MacronomyButtonList[exportButtonCount] then
									local exportLoopButton = MacronomyButtonList[exportButtonCount]
									local btnText = string.gsub(Macronomy[exportLoopClass][exportLoopSpec][exportLoopCategory][exportLoopButton], "\n", "/n/")
									numberOfExportButtons = numberOfExportButtons + 1
									exportText = exportText .. (numberOfExportButtons > 1 and "~~" or "") .. exportLoopButton .. btnText
								end
							end
						end
					end
				end
			end
		end
	end
	
	MacronomyExportFrame_scrollFrame_ebxExport:SetText(exportText)
end
--------------------------------------------------------------------
function Macronomy_ImportText()
	local importText = MacronomyImportFrame_scrollFrame_ebxImport:GetText()
	if importText ~= "" then
		local importClassArray = Macronomy_stringSplit(importText, "~~~~~")
		for importClassCount = 1, getn(importClassArray) do
			local classToImport, strAfterClass = Macronomy_oneWordSplit(importClassArray[importClassCount], ":")
			--DEFAULT_CHAT_FRAME:AddMessage(classToImport)
			local importSpecArray = Macronomy_stringSplit(strAfterClass, "~~~~")
			for importSpecCount = 1, getn(importSpecArray) do
				local specToImport, strAfterSpec = Macronomy_oneWordSplit(importSpecArray[importSpecCount], ":")
				--DEFAULT_CHAT_FRAME:AddMessage("  " .. specToImport)
				local importCategoryArray = Macronomy_stringSplit(strAfterSpec, "~~~")
				for importCategoryCount = 1, getn(importCategoryArray) do
					local categoryToImport, strAfterCategory = Macronomy_oneWordSplit(importCategoryArray[importCategoryCount], ":")
					if not Macronomy[classToImport][specToImport][categoryToImport] then Macronomy_InitCategory(classToImport, specToImport, categoryToImport) end
					--DEFAULT_CHAT_FRAME:AddMessage("    " .. categoryToImport)
					local importButtonArray = Macronomy_stringSplit(strAfterCategory, "~~")
					for importButtonCount = 1, getn(importButtonArray) do
						local importButtonName = string.sub(importButtonArray[importButtonCount],1,2)
						local importButtonContent = string.gsub(string.sub(importButtonArray[importButtonCount],3), "/n/", "\n")
						--if importButtonName == "m1" then DEFAULT_CHAT_FRAME:AddMessage(importButtonName .. ": " .. importButtonContent) end
						Macronomy[classToImport][specToImport][categoryToImport][importButtonName] = importButtonContent
					end
				end
			end
		end
		
		MacronomyButtonUpdate = true
		MacronomyMacroFrame_ebxMacro:SetText(Macronomy[MacronomyByChar.currentClass][MacronomyByChar.currentSpec][MacronomyByChar.currentCategory][MacronomyByChar.currentButton] or "")
	end
end
--------------------------------------------------------------------
function Macronomy_stringSplit(str, delim)
	local outputArray = {}

	while str ~= "" do
		local delimStart = string.find(str, delim)
		outputArray[getn(outputArray) + 1] = (delimStart ~= nil and string.sub(str,1,delimStart - 1) or str)
		str = (delimStart ~= nil and string.sub(str,delimStart + string.len(delim)) or "")
	end
	
	return outputArray
end
--------------------------------------------------------------------
function Macronomy_oneWordSplit(str, delim)
	local delimLoc = string.find(str, delim)
	local rtnStr = (delimLoc ~= nil and string.sub(str, 1, delimLoc - 1) or str)
	local rtnRemainStr = (delimLoc ~= nil and string.sub(str, delimLoc + 1) or "")
	return rtnStr, rtnRemainStr
end
--------------------------------------------------------------------
function Macronomy_disableFilterSection()
	MacronomyFrame_btnCategoryAdd:Disable()
	MacronomyFrame_btnCategoryRemove:Disable()
	UIDropDownMenu_DisableDropDown(MacronomyFrame_btnClass)
	UIDropDownMenu_DisableDropDown(MacronomyFrame_btnSpec)
	UIDropDownMenu_DisableDropDown(MacronomyFrame_btnCategory)
	UIDropDownMenu_DisableDropDown(MacronomyFrame_btnButton)
end
--------------------------------------------------------------------
function Macronomy_enableFilterSection()
	MacronomyFrame_btnCategoryAdd:Enable()
	MacronomyFrame_btnCategoryRemove:Enable()
	UIDropDownMenu_EnableDropDown(MacronomyFrame_btnClass)
	UIDropDownMenu_EnableDropDown(MacronomyFrame_btnSpec)
	UIDropDownMenu_EnableDropDown(MacronomyFrame_btnCategory)
	UIDropDownMenu_EnableDropDown(MacronomyFrame_btnButton)
end
--------------------------------------------------------------------
SLASH_MACRONOMYSHIFT1 = "/macronomyshift"
function SlashCmdList.MACRONOMYSHIFT(msg, editbox)
	Macronomy_shiftCategory()
end

SLASH_MACRONOMYSHOW1 = "/macronomy"
SLASH_MACRONOMYSHOW2 = "/mmy"
function SlashCmdList.MACRONOMYSHOW(msg, editbox)
	if Macronomy.active then
		MacronomyFrame:Hide()
	else
		MacronomyFrame:Show()
		Macronomy_SetDropDowns()
	end
	Macronomy.active = not Macronomy.active
end

--------------------------------------------------------------------

local Macronomy_spellClicked = function(button)
	if IsShiftKeyDown() and MacronomyMacroFrame_ebxMacro:HasFocus() then
		local btnNum = button:GetID() <= 6 and ((button:GetID() * 2) - 1) or ((button:GetID() - 6) * 2)
		MacronomyMacroFrame_ebxMacro:Insert(_G["SpellButton" .. btnNum .. "SpellName"]:GetText())
	end
end
hooksecurefunc("SpellButton_OnModifiedClick", Macronomy_spellClicked)

local Macronomy_equipClicked = function(button)
	if IsShiftKeyDown() and MacronomyMacroFrame_ebxMacro:HasFocus() then
		local itemId = GetInventoryItemID("player", GetInventorySlotInfo(string.sub(button:GetName(),10)))
		if itemId then
			local itemName = GetItemInfo(itemId)
			MacronomyMacroFrame_ebxMacro:Insert(itemName)
		end
	end
end
hooksecurefunc("PaperDollItemSlotButton_OnModifiedClick", Macronomy_equipClicked)

local Macronomy_bagItemClicked = function(button)
	if IsShiftKeyDown() and MacronomyMacroFrame_ebxMacro:HasFocus() then
		local btnName = button:GetName()
		local bagViewer = (string.find(btnName, "ARKINV_Frame") and "ArkInv" or "Default")
		local indexOfItem = string.find(btnName, "Item")
		local bagNum = tonumber(string.sub(btnName, indexOfItem - 1, indexOfItem - 1)) - 1
		local slotNum = tonumber(string.sub(btnName, indexOfItem + 4))
		
		-- Reverse translation of bag slot number for ArkInventory
		--if bagViewer == "ArkInv" then
		--	slotNum = GetContainerNumSlots(bagNum) - slotNum + 1
		--end
		--DEFAULT_CHAT_FRAME:AddMessage(bagNum .. ", " .. slotNum)
		local itemId = GetContainerItemID(bagNum, slotNum)
		if itemId then
			local itemName = GetItemInfo(itemId)
			MacronomyMacroFrame_ebxMacro:Insert(itemName)
		end
	end
end
hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", Macronomy_bagItemClicked)